=begin
Copyright(C) 2010, kyyu
All Rights Reserved
Permission to use, copy, modify, and distribute this software for any purpose
and without fee is hereby granted, provided this notice appears in any copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Name:      		Push/Pull & Group
Version:   		1.0
SU Version:   	7.0
Date:      		05-04-2010

Purpose:		This plugin helps with the drawing of 3D models & puzzles for 1/4" sheet stock.
Description:   	*As Tool:  Push/Pull and group(optional), all with one click of the mouse.
				*Or Select multiple parts and right click context menu
Usage:      	
	# Automatic face picker - just hover mouse over face (I found it useful for selecting faces to delete also)
	# Left mouse button => perform the Push/Pull & Group
	# Right mouse button => toggle group or don't group  (check for message on status bar)
	# Hold down Shift Key => add to selection
	# Hold down Contol Key => reverse the direction of Push/Pull
	# If you change height (default=0.25"), the value is remember for the entire sketchup session.
	# VCB will only except decimal inch values.
	# Can be used from right click context menu, also.
	
Credits:		I learned alot buy studying TBD's Rotix plugin.  Thanks you, for making it!
History:
			1.0 (05-04-2010)  -  first version
	
=end

require 'sketchup.rb'

class KyyuPushPullGroup

   def activate
      @selected = []
	  @face_already = 0
	  @add_selection = 0
	  @nogroup = 0
	  @onRButton = -1
	  @groupyesnotext = '**   Group   **'
	  if $kyyuPushPullGroupThick == nil
			@thickness = -0.25
			else @thickness = $kyyuPushPullGroupThick
	  end
	  Sketchup::set_status_text("Push/Pull & Group Plugin  -  press shift key to add to selection                 #{@groupyesnotext}", SB_PROMPT)
      Sketchup::set_status_text('Thickness', SB_VCB_LABEL)
	  Sketchup::set_status_text("#{-@thickness}", SB_VCB_VALUE)
	 
   end
   
   
 # on MouseMove perform selection
   def onMouseMove(flags, x, y, view)
      ph = view.pick_helper
      found = ph.do_pick(x,y)
      picked = ph.best_picked 

      if (picked != nil)
         # do not add edges/faces to selection
          if (picked.kind_of? Sketchup::Face and (@face_already != 1 or @add_selection == 1))
            Sketchup.active_model.selection.add(picked)
            @selected = picked
			@face_already = 1
          end
		  if (@selected != picked and @add_selection == 0)
		    Sketchup.active_model.selection.clear
			@face_already = 0
		  end		  
      elsif (@add_selection == 0)
          Sketchup.active_model.selection.clear
          @selected = nil
		  @face_already = 0
      end
   end

   # if SHIFT is released, back to normal selection
   def onKeyUp(key, repeat, flags, view)
      @add_selection = 0 if key == CONSTRAIN_MODIFIER_KEY
	  @thickness = -@thickness if key == COPY_MODIFIER_KEY
   end
     
		    
      #process keyboard
   def onKeyDown(key, repeat, flags, view)
	  case key
         when CONSTRAIN_MODIFIER_KEY then @add_selection = 1 if (repeat == 1)
		 when COPY_MODIFIER_KEY then @thickness = -@thickness if (repeat == 1)
      end
	end
	
		  # process mouse
   def onRButtonDown(flags, x, y, view)
		@onRButton = -@onRButton
		if @onRButton == 1
		   @nogroup = 1
		   @groupyesnotext = '**** NO Group ****'
		end
		if @onRButton == -1
			@nogroup = 0
			@groupyesnotext = '**   Group   **'
		end
		Sketchup::set_status_text("Push/Pull & Group Plugin  -  press shift key to add to selection                 #{@groupyesnotext}", SB_PROMPT)
   end
   
	      # process mouse
   def onLButtonDown(flags, x, y, view)
   		model = Sketchup.active_model
		ent = model.entities
		sel = model.selection
				Sketchup.active_model.start_operation "PushPull Group" 		
			sel.each do |e|			
				e.pushpull (@thickness)
				if @nogroup == 0 then ent.add_group e.all_connected	end
			end	
				 Sketchup.active_model.commit_operation					 
   end
   

     # VCB
   def onUserText(text,view)
      begin
         value = text.to_f
      rescue
         # Error parsing the text
         UI.beep
         value = nil
         Sketchup::set_status_text "", SB_VCB_VALUE
      end
      return if !value
   
     @thickness = -value
	 $kyyuPushPullGroupThick = -value
	 Sketchup::set_status_text("#{-@thickness}", SB_VCB_VALUE)
   end

   
end		# end class KyyuPushPullGroup

	# add context menu item, so can use native select tool
def kyyu_context_pushpullgroup
    model = Sketchup.active_model
	ent = model.entities
	sel = model.selection
    if $kyyuPushPullGroupThick == nil
		thick = -0.25
		else thick = $kyyuPushPullGroupThick
	end
			Sketchup.active_model.start_operation "context PushPull Group" 		
	sel.each do |e|
		if e.is_a? Sketchup::Face
			e.pushpull (thick)
			ent.add_group e.all_connected
		end
	end	
			Sketchup.active_model.commit_operation	
end

#----------------------------------------------------------------------------
# add context menu items
   
 if( not file_loaded?('ky_PushPull and Group.rb') )
	if $kyyu_context_submenu == 1
		UI.add_context_menu_handler do |menu|
			$kyyu_ConName.add_item('PushPull and Group') { kyyu_context_pushpullgroup }
		end
	else
		UI.add_context_menu_handler do |menu|
			$kyyu_ConName=menu.add_submenu('[kyyu]')
			$kyyu_ConName.add_item('PushPull and Group') { kyyu_context_pushpullgroup }
		end
		$kyyu_context_submenu = 1
	end
end  
   
# UI.add_context_menu_handler do |context_menu| 
    # context_menu.add_item("[ky] PushPull and Group") { kyyu_context_pushpullgroup }  
# end



#----------------------------------------------------------------------------
# add plugin menu items

if( not file_loaded?('ky_PushPull and Group.rb') )
	if $kyyu_submenu == 1
		$kyyu_PlugName.add_item('PushPull and Group') { Sketchup.active_model.select_tool KyyuPushPullGroup.new }
	else
		$kyyu_PlugName=UI.menu('Plugins').add_submenu('[kyyu]')
		$kyyu_PlugName.add_item('PushPull and Group') { Sketchup.active_model.select_tool KyyuPushPullGroup.new }
		$kyyu_submenu = 1
	end
end

# if( not file_loaded?('ky_PushPull and Group.rb') )
  # UI.menu('Plugins').add_item('[ky] PushPull and Group') { Sketchup.active_model.select_tool KyyuPushPullGroup.new }
# end

file_loaded 'ky_PushPull and Group.rb'


